/*********************************************************************
 * Sequence for Desktop Station main
 *
 * Copyright (C) 2015 Yaasan
 *
 */

#include <Arduino.h>
#include <EEPROM.h>
#include <avr/wdt.h>
#include "DSSequence.h"

#define DEBUG 

#define		BTN_NUM5	0b0000000000000001 //SW09
#define		BTN_NUM1	0b0000000000000010 //SW02
#define		BTN_NUM9	0b0000000000000100 //SW13
#define		BTN_NUM4	0b0000000000001000 //SW05
#define		BTN_CLR		0b0000000000010000 //SW14
#define		BTN_NUM0	0b0000000000100000 //SW01
#define		BTN_NUM2	0b0000000001000000 //SW03
#define		BTN_NUM8	0b0000000010000000 //SW12
#define		BTN_ESC		0b0000000100000000 //SW06
#define		BTN_UP		0b0000001000000000 //SW15
#define		BTN_NUM6	0b0000010000000000 //SW06
#define		BTN_NUM3	0b0000100000000000 //SW04
#define		BTN_NUM7	0b0001000000000000 //SW11
#define		BTN_ENT		0b0010000000000000 //SW07
#define		BTN_DN		0b0100000000000000 //SW08


#define		MAX_MAINMENU	11


DSSequence::DSSequence()
{
	
	for( int i = 0; i < 128; i++)
	{
		gAcc_FuncBuf[i] = 0;
	}
	
	gMode = MODE_MAINMENU;
	gNo_CVReadMenu = 0;
	gNo_CVWriteMenu = 0;
	gNo_CVReadWriteMenu = 0;
	gNo_AddrChkMenu = 0;
	gNo_AddrWriteMenu = 0;
	gNo_AccCtrlMenu = 0;
	gNo_LocCtrlMenu = 0;
	gNo_FactoryResetMenu = 0;
	gNo_ManCtrlMenu = 0;
	gNo_ConfigMenu = 0;
	gNo_NSignalMenu = 0;
	
	
	gAcc_Addr = 1;
	gAcc_Dir = 0;
	gLoc_Addr = 3;
	gLoc_Func = 0;
	gLoc_Dir = 0;
	gLoc_Spd = 0;
	
	gCfgReadSpeed = 0;
	gCfgReadPower = 0;
	gCfgThreshold = 8;
	
	CVNo = 1;
	CVValue = 0;
	Address = 0;
	
	gLastManID = 0;
	
	SignalAddr[0] = 0;
	SignalAddr[1] = 0;
	SignalAddr[2] = 0;
	SignalAddr[3] = 0;
	SignalAddr[4] = 0;
	SignalAddr[5] = 0;
	
	loadEEPROM();
	
	gDScmd.mCmd = 0;
	
}

PackedDScmd DSSequence::GetCommad()
{
	return gDScmd;
}


void DSSequence::loadEEPROM()
{
	
	if( EEPROM.read(0) == 0xBB)
	{
		
		gCfgReadSpeed = EEPROM.read(1);
		gCfgThreshold = EEPROM.read(2);
		
		if( gCfgReadSpeed > 1)
		{
			gCfgReadSpeed = 0;
		}
		
		if( gCfgThreshold > 64)
		{
			gCfgThreshold = 8;
		}
		
	}	
	
}

void DSSequence::saveEEPROM(byte inMode)
{
	
	if( EEPROM.read(0) == 0xBB)
	{
		
		switch(inMode)
		{
		case 0:
			EEPROM.write(1, gCfgReadSpeed);
			break;
			
		case 1:
			EEPROM.write(2, gCfgThreshold);
			break;
			
		}
	}
	else
	{
		//初期化
		EEPROM.write(0, 0xBB);
		
		//設定
		EEPROM.write(1, gCfgReadSpeed);
		EEPROM.write(2, gCfgThreshold);

		
	}
	
}

word DSSequence::getButtonStatus()
{
	
	word aButton_val = 0;
	
	byte aTempBtnA = 0;
	byte aTempBtnB = 0;
	byte aTempBtnC = 0;
	
	unsigned short aButtonA = analogRead(PORT_BTNA);
	unsigned short aButtonB = analogRead(PORT_BTNB);
	unsigned short aButtonC = analogRead(PORT_BTNC);
	
	//Get button status
	aTempBtnA = GetButtonsFromAD(aButtonA);
	aTempBtnB = GetButtonsFromAD(aButtonB);
	aTempBtnC = GetButtonsFromAD(aButtonC);
	
	//Check and process the buttons
	aButton_val = ProcessCheck(aTempBtnA, aTempBtnB, aTempBtnC);
	
	return aButton_val;
	
}







byte DSSequence::Interval()
{
	bool aChange = false;
	word aButton_val = 0;
	
	/* 更新フラグをクリア */
	gDisplayFlag = 0;
	
	//Input Buttons
	aButton_val = getButtonStatus();
	
	//Sequence main
	
	switch(gMode)
	{
	case MODE_MAINMENU:
		interval_Main(aButton_val);
		break;
		
	case MODE_CVREAD:
		interval_ReadCV(aButton_val);
		break;
		
	case MODE_CVWRITE:
		interval_WriteCV(aButton_val);
		break;
		
	case MODE_CVREADWRITE:
		interval_ReadWriteCV(aButton_val);
		break;
		
	case MODE_NUCKYSIGNAL:
		interval_Signal(aButton_val);
		break;
		
	case MODE_CHECKADDRESS:
		interval_CheckAddress(aButton_val);
		break;
		
	case MODE_WRITEADDRESS:
		interval_WriteAddress(aButton_val);
		break;
		
	case MODE_LOCCTRL:
		interval_LocCtrl(aButton_val);
		break;
		
	case MODE_ACCCTRL:
		interval_AccCtrl(aButton_val);
		break;
		
	case MODE_MANUFACTURE:
		interval_Manufacture(aButton_val);
		break;
		
	case MODE_FACTORYRESET:
		interval_FactoryReset(aButton_val);
		break;
		
	case MODE_ERRSTATUS:
		interval_ErrorStatus(aButton_val);
		break;		
		
	case MODE_CONFIG:
		interval_Config(aButton_val);
		break;		
	}
	
	//Always run event
	interval_always(aButton_val);
	
	/*
	if( gButtonBuf != aButton_val)
	{
		//Serial.print("Button=");
		//Serial.println(aButton_val, BIN);
	}	
	*/
	
	//Store previous values
	gButtonBuf = aButton_val;
	
	return gDisplayFlag;
}

void DSSequence::interval_always(word inButtonStatus)
{
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			
			
			
		}
	}	
	
}

void DSSequence::interval_Main(word inButtonStatus)
{
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			//モード切替
			gMode = gNo_MainMenu + 2;
			
#ifdef DEBUG
			Serial.print("Next Mode=");
			Serial.println(gMode, DEC);
#endif
			
			
			if ( (gMode == MODE_LOCCTRL) || (gMode == MODE_ACCCTRL))
			{
				
#ifdef DEBUG
				Serial.println("Power On (CTRL mode)");
#endif
				
				//パワーオン
				ChangedPowerStatus(1);
				
			}
			
			
			//表示処理
			gDisplayFlag |= 1;
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_UP) == true )
		{
			
			if( gNo_MainMenu <= 0)
			{
				gNo_MainMenu = 0;
			}
			else
			{
				gNo_MainMenu--;
			}
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_DN) == true )
		{
			
			if( gNo_MainMenu >= MAX_MAINMENU)
			{
				gNo_MainMenu = MAX_MAINMENU;
			}
			else
			{
				gNo_MainMenu++;
			}
			
			//表示処理
			gDisplayFlag |= 1;
			
		}

		

	}	
}

bool DSSequence::editNums(word inButtonStatus, word *iopNum, word inMax)
{
	word aNum = *iopNum;
	bool aRet = false;
	
	if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM1) == true )
	{
		aNum = aNum * 10 + 1;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM2) == true )
	{
		aNum = aNum * 10 + 2;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM3) == true )
	{
		aNum = aNum * 10 + 3;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM4) == true )
	{
		aNum = aNum * 10 + 4;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM5) == true )
	{
		aNum = aNum * 10 + 5;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM6) == true )
	{
		aNum = aNum * 10 + 6;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM7) == true )
	{
		aNum = aNum * 10 + 7;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM8) == true )
	{
		aNum = aNum * 10 + 8;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM9) == true )
	{
		aNum = aNum * 10 + 9;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM0) == true )
	{
		aNum = aNum * 10 + 0;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_CLR) == true )
	{
		aNum = 0;
		aRet = true;
	}	
	
	//Check max value
	if( aNum >= inMax)
	{
		aNum = inMax;
		aRet = true;
	}
	
	
	*iopNum = aNum;
	
	return aRet;
}

bool DSSequence::editSpds(word inButtonStatus, word *iopNum)
{
	word aNum = *iopNum;
	bool aRet = false;
	
	if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM1) == true )
	{
		aNum = 1;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM2) == true )
	{
		aNum = 2;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM3) == true )
	{
		aNum = 3;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM4) == true )
	{
		aNum = 4;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM5) == true )
	{
		aNum = 5;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM6) == true )
	{
		aNum = 6;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM7) == true )
	{
		aNum = 7;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM8) == true )
	{
		aNum = 8;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM9) == true )
	{
		aNum = 9;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM0) == true )
	{
		aNum = 0;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_CLR) == true )
	{
		aNum = 0;
		aRet = true;
	}	
	
	//Check max value
	if( aNum >= 10)
	{
		aNum = 9;
		aRet = true;
	}
	
	
	*iopNum = aNum;
	
	return aRet;
}


void DSSequence::interval_Manufacture(word inButtonStatus)
{
	unsigned char aValue;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			if( gNo_ManCtrlMenu == 0)
			{
				
				if( CVNo > 0)
				{
					gNo_ManCtrlMenu = 1;
					
					//読み込み処理実行
					gDScmd.mParam1 = 8;
					gDScmd.mCmd = DSCMD_CVREAD;
					gDScmd.mWait = 1;
					
					//実行
					CVExeFunc(&aValue, 1);
					
					//結果表示
					gNo_ManCtrlMenu = aValue;
					
					//表示処理
					gDisplayFlag |= 1;
				}
			}
			else
			{
				//メニューに戻る
				gMode = MODE_MAINMENU;
				gNo_ManCtrlMenu = 0;
				
				//表示処理
				gDisplayFlag |= 1;
			}
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_ManCtrlMenu = 0;
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else
		{
			

		}		
	}	
}


void DSSequence::interval_ReadCV(word inButtonStatus)
{
	unsigned char aValue;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			if( gNo_CVReadMenu == 0)
			{
				
				if( CVNo > 0)
				{
					gNo_CVReadMenu = 1;
					
					//読み込み処理実行
					gDScmd.mParam1 = CVNo;
					gDScmd.mCmd = DSCMD_CVREAD;
					gDScmd.mWait = 1;
					
					//実行
					CVExeFunc(&aValue, 1);
					
					//結果表示
					gNo_CVReadMenu = aValue;
					
					//表示処理
					gDisplayFlag |= 1;
				}
			}
			else
			{
				//メニューに戻る
				gMode = MODE_MAINMENU;
				gNo_CVReadMenu = 0;
				
				//表示処理
				gDisplayFlag |= 1;
			}
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_CVReadMenu = 0;
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_UP) == true )
		{
			
			//gNo_CVReadMenu = 0;
			
			//表示処理
			//gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_DN) == true )
		{
			
			//gNo_CVReadMenu = 1;
			
			//表示処理
			//gDisplayFlag |= 1;
			
		}
		else
		{
			
			if( gNo_CVReadMenu == 0)
			{
				if( editNums(inButtonStatus, &CVNo, 1024) == true)
				{
					//表示処理
					gDisplayFlag |= 1;
				}
			}
		}		
	}	
}

void DSSequence::interval_WriteCV(word inButtonStatus)
{
	unsigned char aValue;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			if( gNo_CVWriteMenu == 0)
			{
				gNo_CVWriteMenu = 1;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			else if( gNo_CVWriteMenu == 1)
			{
				if( CVNo > 0)
				{
					gNo_CVWriteMenu = 2;
					
					//読み込み処理実行
					gDScmd.mParam1 = CVNo;
					gDScmd.mParam2 = CVValue;
					gDScmd.mCmd = DSCMD_CVWRITE;
					gDScmd.mWait = 1;
					
					//実行
					CVExeFunc(&aValue, 1);
					
					//結果表示
					gNo_CVWriteMenu = aValue;
					
					//表示処理
					gDisplayFlag |= 1;
				}
			}
			else
			{
				//メニューに戻る
				gMode = MODE_MAINMENU;
				gNo_CVWriteMenu = 0;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_CVWriteMenu = 0;
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_UP) == true )
		{
			
			gNo_CVWriteMenu = 0;
			
			//表示処理
			//gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_DN) == true )
		{
			
			gNo_CVWriteMenu = 1;
			
			//表示処理
			//gDisplayFlag |= 1;
			
		}
		else
		{
			
			if( gNo_CVWriteMenu == 0)
			{
				if( editNums(inButtonStatus, &CVNo, 1024) == true)
				{
					//表示処理
					gDisplayFlag |= 1;
				}

			}
			else if( gNo_CVWriteMenu == 1)
			{
				if( editNums(inButtonStatus, &CVValue, 255) == true)
				{
					//表示処理
					gDisplayFlag |= 1;
				}

			}
		}		
	}	
}

void DSSequence::interval_ReadWriteCV(word inButtonStatus)
{
	unsigned char aValue;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			if( gNo_CVReadWriteMenu == 0)
			{
				gNo_CVReadWriteMenu = 1;
				
				//読み込み処理実行
				gDScmd.mParam1 = CVNo;
				gDScmd.mCmd = DSCMD_CVREAD;
				gDScmd.mWait = 1;
				
				//実行
				CVExeFunc(&aValue, 1);
				
				if( aValue == 2)
				{
					gNo_CVReadWriteMenu = 2;
					
				}
				else
				{
					gNo_CVReadWriteMenu = 6;
					
				}
				
				
				//表示処理
				gDisplayFlag |= 1;
			}
			else if( gNo_CVReadWriteMenu == 2)
			{
				
				if( CVNo > 0)
				{
					gNo_CVReadWriteMenu = 3;
					
					//書き込み処理実行
					gDScmd.mParam1 = CVNo;
					gDScmd.mParam2 = CVValue;
					gDScmd.mCmd = DSCMD_CVWRITE;
					gDScmd.mWait = 1;
					
					//実行
					CVExeFunc(&aValue, 1);
					
					//結果表示
					if( aValue == 3)
					{
						gNo_CVReadWriteMenu = 4;
						
					}
					else
					{
						gNo_CVReadWriteMenu = 5;
						
					}
					
					//表示処理
					gDisplayFlag |= 1;
				}
			}
			else
			{
				//メニューに戻る
				gMode = MODE_MAINMENU;
				gNo_CVReadWriteMenu = 0;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_CVReadWriteMenu = 0;
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_UP) == true )
		{
			
			gNo_CVReadWriteMenu = 0;
			
			//表示処理
			//gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_DN) == true )
		{
			
			gNo_CVReadWriteMenu = 1;
			
			//表示処理
			//gDisplayFlag |= 1;
			
		}
		else
		{
			
			if( gNo_CVReadWriteMenu == 0)
			{
				if( editNums(inButtonStatus, &CVNo, 1023) == true)
				{
					//表示処理
					gDisplayFlag |= 1;
				}

			}
			else if( gNo_CVReadWriteMenu == 2)
			{
				if( editNums(inButtonStatus, &CVValue, 255) == true)
				{
					//表示処理
					gDisplayFlag |= 1;
				}

			}
		}		
	}	
}

void DSSequence::interval_Signal(word inButtonStatus)
{

	unsigned char aValue;
	unsigned char aUpperVal;
	unsigned char aLowerVal;
	unsigned char aWriteOk = 0;
	word aTempAddr;
	word aAddr;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			if( gNo_NSignalMenu <= 4)
			{
				gNo_NSignalMenu++;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			else if( gNo_NSignalMenu == 5)
			{
				
				gNo_NSignalMenu = 6;
				
				
				for( int i = 0; i < 6; i++)
				{
					
					if( SignalAddr[i] > 0)
					{
#ifdef DEBUG
						Serial.print("Address");
						Serial.println(i + 1);
#endif
						
						//アドレスを-1する（0始まり)
						aAddr = SignalAddr[i] - 1;
						
						//CV値取得
						aTempAddr = ((aAddr) >> 2) + 1;
						aUpperVal = (0b10000000 | (aTempAddr & 0b00111111));//10AAAAAA 
						aLowerVal = (0b10001000 | ((((~aTempAddr) & 0b111000000) >> 2) & 0xFF)) | (((aAddr % 4) & 0b11) << 1);//1AAACDDD
						
						
						//書き込み処理実行
						gDScmd.mParam1 = 11 + (i * 2 + 0);
						gDScmd.mParam2 = aUpperVal;
						gDScmd.mCmd = DSCMD_CVWRITE;
						gDScmd.mWait = 1;
						
						//実行
						CVExeFunc(&aValue, 1);
						
						if( aValue == 3)
						{
							aWriteOk++;
						}
						
						//書き込み処理実行
						gDScmd.mParam1 = 11 + (i * 2 + 1);
						gDScmd.mParam2 = aLowerVal;
						gDScmd.mCmd = DSCMD_CVWRITE;
						gDScmd.mWait = 1;
						
						//実行
						CVExeFunc(&aValue, 0);
						
						if( aValue == 3)
						{
							aWriteOk++;
						}
						
#ifdef DEBUG
						Serial.print("CV");
						Serial.print(11 + (i * 2 + 0));
						Serial.print("=");
						Serial.println(aUpperVal);
						
						Serial.print("CV");
						Serial.print(11 + (i * 2 + 1));
						Serial.print("=");
						Serial.println(aLowerVal);
#endif						
					}
					else
					{
						//CVを書き換えない時はOkだったとする。
						aWriteOk += 2;
					}
					
					/* Reset watchdog timer. */
					wdt_reset(); 
				
				}
				
				
				
				if( aWriteOk >= 10)
				{
					//結果表示
					gNo_NSignalMenu = 7;
				}
				else
				{
					//結果表示
					gNo_NSignalMenu = 99;
				}
				
				//表示処理
				gDisplayFlag |= 1;
			}
			else
			{
				//メニューに戻る
				gMode = MODE_MAINMENU;
				gNo_NSignalMenu = 0;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_NSignalMenu = 0;
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_UP) == true )
		{
			
			if( gNo_NSignalMenu > 0)
			{
				gNo_NSignalMenu--;
			}
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_DN) == true )
		{
			if( gNo_NSignalMenu <= 4)
			{
				gNo_NSignalMenu++;
			}
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else
		{
			
			if( gNo_NSignalMenu <= 5)
			{
				if( editNums(inButtonStatus, &SignalAddr[gNo_NSignalMenu], 999) == true)
				{
					//表示処理
					gDisplayFlag |= 1;
				}
			}
		}		
	}	
	
}

void DSSequence::interval_CheckAddress(word inButtonStatus)
{

	unsigned char aValue;
	unsigned char aCV29;
	unsigned char aCV01;
	unsigned char aCV17;
	unsigned char aCV18;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			if( gNo_AddrChkMenu == 0)
			{
				gNo_AddrChkMenu = 1;
				
				//読み込み処理実行
				gDScmd.mParam1 = 29;
				gDScmd.mCmd = DSCMD_CVREAD;
				gDScmd.mWait = 1;
				gCfgReadPower = 1;
				
				//実行
				CVExeFunc(&aValue, 1);
				
				if( aValue == 2)
				{
					/* Reset watchdog timer. */
					wdt_reset(); 
					
					aCV29 = CVValue;
					gCfgReadPower = 3;
					
#ifdef DEBUG
					Serial.print("CV29=");
					Serial.println(aCV29);
#endif
					
					if( (aCV29 & 0b100000) > 0  )
					{
						
						//読み込み処理実行
						gDScmd.mParam1 = 17;
						gDScmd.mCmd = DSCMD_CVREAD;
						gDScmd.mWait = 1;
						
						//実行
						CVExeFunc(&aValue, 0);
						
						if( aValue == 2)
						{
							aCV17 = CVValue;
							
#ifdef DEBUG
							Serial.print("CV17=");
							Serial.println(aCV17);
#endif
							
							//読み込み処理実行
							gDScmd.mParam1 = 18;
							gDScmd.mCmd = DSCMD_CVREAD;
							gDScmd.mWait = 1;
							
							//実行
							CVExeFunc(&aValue, 0);
							
							if( aValue == 2)
							{
								aCV18 = CVValue;
								
#ifdef DEBUG
								Serial.print("CV18=");
								Serial.println(aCV18);
#endif
								
								//アドレス計算
								Address = aCV18 + ((aCV17 & 0b00111111) << 8);
								
								gNo_AddrChkMenu = 2;
								
							}
							else
							{
								gNo_AddrChkMenu = 99;
							}
							
							
						}
						else
						{
							gNo_AddrChkMenu = 99;
						}
						
					}
					else
					{
						//読み込み処理実行
						gDScmd.mParam1 = 1;
						gDScmd.mCmd = DSCMD_CVREAD;
						gDScmd.mWait = 1;
						
						//実行
						CVExeFunc(&aValue, 0);
						
						if( aValue == 2)
						{
							aCV01 = CVValue;
							
							//アドレス計算
							Address = aCV01;
							
							gNo_AddrChkMenu = 2;
						}
						else
						{
							gNo_AddrChkMenu = 99;
						}
						
					}
					
					
				}
				else
				{
					gNo_AddrChkMenu = 99;
					
				}
				
				
				//電源OFF
				ChangedPowerStatus(0);
				gCfgReadPower = 0;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			else
			{
				//メニューに戻る
				gMode = MODE_MAINMENU;
				gNo_AddrChkMenu = 0;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_AddrChkMenu = 0;
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else
		{

		}		
	}	
	
}

void DSSequence::interval_FactoryReset(word inButtonStatus)
{

	unsigned char aValue;
	unsigned char aCV08;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			if( gNo_FactoryResetMenu == 0)
			{
				/* メーカー名表示、リセット確認 */
				gNo_FactoryResetMenu = 1;
				
				//読み込み処理実行
				gDScmd.mParam1 = 8;
				gDScmd.mCmd = DSCMD_CVREAD;
				gDScmd.mWait = 1;
				gCfgReadPower = 1;
				
				//実行
				CVExeFunc(&aValue, 1);
				
				if( aValue == 2)
				{
					/* Reset watchdog timer. */
					wdt_reset(); 
					
					aCV08 = CVValue;
					gCfgReadPower = 3;
					gLastManID = CVValue;
					
#ifdef DEBUG
					Serial.print("CV08=");
					Serial.println(aCV08);
#endif
					
					/* メーカー名表示、リセット確認 */
					gNo_FactoryResetMenu = 2;
					
				}
				else
				{
					gNo_FactoryResetMenu = 99;
					
				}
				
				//表示処理
				gDisplayFlag |= 1;
			}
			else if( gNo_FactoryResetMenu == 2)
			{
				
				/* CV8にリセット書き込み（メーカーによる) */
				
				gDScmd.mParam1 = 8;
				
				switch(gLastManID)
				{
				case 99:
					//Lenz
					gDScmd.mParam2 = 33;
					break;
					
				case 108:
				case 140:
					//Smile Type-A
					gDScmd.mParam2 = 1;
					break;
					
				case 153:
				case 11:
					gDScmd.mParam2 = 2;
					break;
					
				case 103:
					gDScmd.mParam2 = 103;
					break;
					
				case 101:
					//Bachmann
					gDScmd.mParam2 = 141;
					break;
					
				case 161:
					gDScmd.mParam2 = 32;
					break;
					
				case 117:
					//cT
					gDScmd.mParam1 = 1;
					gDScmd.mParam2 = 0;
					break;
					
				case 27:
				case 159:
					//MTH, LGB
					gDScmd.mParam1 = 55;
					gDScmd.mParam2 = 55;
					break;
					
				case 143:
					gDScmd.mParam1 = 125;
					gDScmd.mParam2 = 1;
					break;
					
				case 57:
					gDScmd.mParam1 = 255;
					gDScmd.mParam2 = 123;
					break;
					
				case 85:
					gDScmd.mParam1 = 59;
					gDScmd.mParam2 = 1;
					break;

				default:
					gDScmd.mParam2 = 8;
					break;
				}
			
				gDScmd.mCmd = DSCMD_CVWRITE;
				gDScmd.mWait = 1;
				
				gNo_FactoryResetMenu = 4;
			
				//実行
				CVExeFunc(&aValue, 1);
				
				if( aValue == 3)
				{
					//aOk |= 0b0001;
					gNo_FactoryResetMenu = 5;
				}
				else
				{
					gNo_FactoryResetMenu = 6;
				}
				
				
				//電源OFF
				ChangedPowerStatus(0);
				
			
				//表示処理
				gDisplayFlag |= 1;
			}
			else
			{
				//メニューに戻る
				gMode = MODE_MAINMENU;
				gNo_FactoryResetMenu = 0;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_FactoryResetMenu = 0;
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else
		{

		}		
	}	
	
}

void DSSequence::interval_WriteAddress(word inButtonStatus)
{

	unsigned char aValue;
	unsigned char aOk = 0;
	unsigned char aCV29;
	unsigned char aCV01;
	unsigned char aCV17;
	unsigned char aCV18;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			
			if( Address > 0)
			{
				
				
				if( gNo_AddrWriteMenu == 0)
				{
					//アドレス自動設定
					if( Address > 100)
					{
						aCV17 = 0b11000000 | (Address >> 8);
						aCV18 = 0xFF & Address;
						aCV01 = 0;
					}
					else
					{
						aCV01 = Address;
					}
					
					
					//表示切替
					gNo_AddrWriteMenu = 1;
					
					//読み込み処理実行
					gDScmd.mParam1 = 29;
					gDScmd.mCmd = DSCMD_CVREAD;
					gDScmd.mWait = 1;
					
					//実行
					CVExeFunc(&aValue, 1);
					
					if( aValue == 2)
					{
						/* Reset watchdog timer. */
						wdt_reset(); 
						
						//表示切替
						gNo_AddrWriteMenu = 2;
						
						if( (aCV01 != 0) && ( (CVValue & 0b100000) != 0))
						{
							//SHORT Settings
							aCV29 = CVValue & 0b11011111;
							
							//CV29書込実行
							gDScmd.mParam1 = 29;
							gDScmd.mParam2 = aCV29;
							gDScmd.mCmd = DSCMD_CVWRITE;
							gDScmd.mWait = 1;
							
							//実行
							CVExeFunc(&aValue, 1);
							
							if( aValue == 3)
							{
								aOk |= 0b0001;
							}
						}
						else if( (aCV01 == 0) && ( (CVValue & 0b100000) == 0))
						{
							//LONG Settings
							aCV29 = CVValue | 0b100000;
							
							//CV29書込実行
							gDScmd.mParam1 = 29;
							gDScmd.mParam2 = aCV29;
							gDScmd.mCmd = DSCMD_CVWRITE;
							gDScmd.mWait = 1;
							
							//実行
							CVExeFunc(&aValue, 1);
							
							if( aValue == 3)
							{
								aOk |= 0b0001;
							}
							
						}
						else
						{
							aOk |= 0b0001;
						}
						
#ifdef DEBUG
						Serial.print("CV29=");
						Serial.println(aCV29);
#endif
						
						if( aCV01 > 0)
						{
							//Short
							
							//CV01書込実行
							gDScmd.mParam1 = 1;
							gDScmd.mParam2 = aCV01;
							gDScmd.mCmd = DSCMD_CVWRITE;
							gDScmd.mWait = 1;
							
							//実行
							CVExeFunc(&aValue, 1);
							
							if( aValue == 3)
							{
								aOk |= 0b1110;
							}
							
						}
						else
						{
							//Long
							
							//CV17書込実行
							gDScmd.mParam1 = 17;
							gDScmd.mParam2 = aCV17;
							gDScmd.mCmd = DSCMD_CVWRITE;
							gDScmd.mWait = 1;
							
							//実行
							CVExeFunc(&aValue, 1);
							
							if( aValue == 3)
							{
								aOk |= 0b0110;
							}
							
							//CV18書込実行
							gDScmd.mParam1 = 18;
							gDScmd.mParam2 = aCV18;
							gDScmd.mCmd = DSCMD_CVWRITE;
							gDScmd.mWait = 1;
							
							//実行
							CVExeFunc(&aValue, 0);
							
							if( aValue == 3)
							{
								aOk |= 0b1000;
							}
							
						}
						
						if( aOk == 0b1111)
						{
							gNo_AddrWriteMenu = 3;
							
						}
						else
						{
							gNo_AddrWriteMenu = 4;
							
						}
						
						
					}
					else
					{
						//CV29 エラー
						gNo_AddrWriteMenu = 99;
						
					}
					
					//表示処理
					gDisplayFlag |= 1;
				}
				else
				{
					//メニューに戻る
					gMode = MODE_MAINMENU;
					gNo_AddrWriteMenu = 0;
					
					//表示処理
					gDisplayFlag |= 1;
				}
			}
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_AddrWriteMenu = 0;
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else
		{
			if( gNo_AddrWriteMenu == 0)
			{
				if( editNums(inButtonStatus, &Address, 9999) == true)
				{
					//表示処理
					gDisplayFlag |= 1;
				}
			}
		}		
	}	
	
}

void DSSequence::interval_Config(word inButtonStatus)
{
	unsigned char aValue;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			
			switch( gNo_ConfigMenu)
			{
			case 0:
				//cv read speed:
				
				if( gCfgReadSpeed == 0)
				{
					gCfgReadSpeed = 1;
				}
				else
				{
					gCfgReadSpeed = 0;
				}
				
				//保存処理
				saveEEPROM(0);
				
				//表示処理
				gDisplayFlag |= 1;
				
					break;
			
			case 1:
				//cv threshold:
				
				if( gCfgThreshold > 64)
				{
					gCfgThreshold = 64;
				}
				
				//保存処理
				saveEEPROM(1);
				
				//表示処理
				gDisplayFlag |= 1;
				
				break;
			
			
			
			default:
			case 2:
				//メニューに戻る
				gMode = MODE_MAINMENU;
				gNo_CVReadMenu = 0;
				
				//表示処理
				gDisplayFlag |= 1;
				
				break;
				
			}
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_CVReadMenu = 0;
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_UP) == true )
		{
			
			if( gNo_ConfigMenu > 0)
			{
				gNo_ConfigMenu--;
			}
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_DN) == true )
		{
			
			if( gNo_ConfigMenu < 2)
			{
				gNo_ConfigMenu++;
			}
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else
		{
			
			if( gNo_ConfigMenu == 1)
			{
				if( editNums(inButtonStatus, &gCfgThreshold, 64) == true)
				{
					//表示処理
					gDisplayFlag |= 1;
				}
			}
		}
	}	
}

void DSSequence::interval_LocCtrl(word inButtonStatus)
{

	unsigned char aValue;
	unsigned char aUpperVal;
	unsigned char aLowerVal;
	unsigned char aWriteOk = 0;
	word aTempAddr;
	word aAddr;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			
			switch(gNo_LocCtrlMenu)
			{
			case 0:
				//Address
				
				if( gLoc_Addr == 0)
				{
					gNo_LocCtrlMenu = 0;
				}
				else
				{
					//Go to Speed/FWD/REV
					gNo_LocCtrlMenu = 1;
				}
				
				break;
				
			case 1:
				//Speed/FWD/REV
				
				if( gLoc_Dir > 0)
				{
					gLoc_Dir = 0;
				}
				else
				{
					gLoc_Dir = 1;
				}
				
				//セット
				ChangedTargetDir(gLoc_Addr, gLoc_Dir);
				
				break;
				
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
			case 7:
			case 8:
			case 9:
			case 10:
			case 11:
			case 12:
			case 13:
			case 14:
			case 15:
			case 16:
			case 17:
			case 18:
			case 19:
			case 20:
			case 21:
			case 22:
			case 23:
			case 24:
			case 25:
			case 26:
			case 27:
			case 28:
			case 29:
			case 30:
				//Function
				
				uint32_t aFuncNo = (gNo_LocCtrlMenu - 2);
				byte aFunc = (gLoc_Func >> aFuncNo) & 1;
				
				if( aFunc == 0)
				{
					gLoc_Func = gLoc_Func | (1UL << aFuncNo);
					aFunc = 1;
				}
				else
				{
					gLoc_Func = gLoc_Func & ~(1UL << aFuncNo);
					aFunc = 0;
				}
				
				//セット
				ChangedTargetFunc(gLoc_Addr ,aFuncNo + 1, aFunc);
				
				
				break;
				
			}
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_LocCtrlMenu = 0;
			
			//パワーオフ
			ChangedPowerStatus(0);
			
#ifdef DEBUG
			Serial.println("Power Off (CTRL mode)");
#endif
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_UP) == true )
		{
			
			if( gNo_LocCtrlMenu > 0)
			{
				gNo_LocCtrlMenu--;
			}
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_DN) == true )
		{
			if( gNo_LocCtrlMenu < 30)
			{
				if( gLoc_Addr == 0)
				{
					gNo_LocCtrlMenu = 0;
				}
				else
				{
					gNo_LocCtrlMenu++;
				}
			}
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else
		{
			
			if( gNo_LocCtrlMenu == 0)
			{
				if( editNums(inButtonStatus, &gLoc_Addr, 9999) == true)
				{
					
					//表示処理
					gDisplayFlag |= 1;
				}
			}
			else if(gNo_LocCtrlMenu == 1)
			{
				if( editSpds(inButtonStatus, &gLoc_Spd) == true)
				{
					//表示処理
					gDisplayFlag |= 1;
					
					//セット
					ChangedTargetSpeed(gLoc_Addr, gLoc_Spd);
				}
			}
			else
			{
				
			}
		}
	}	
	
}

void DSSequence::interval_AccCtrl(word inButtonStatus)
{

	unsigned char aValue;
	unsigned char aUpperVal;
	unsigned char aLowerVal;
	unsigned char aWriteOk = 0;
	word aTempAddr;
	word aAddr;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			
			
			if( gAcc_Dir == 0)
			{
				gAcc_Dir = 1;
				
				if( gAcc_Addr < 1024 )
				{
					gAcc_FuncBuf[gAcc_Addr >> 3] |= (1UL << (gAcc_Addr & 7));
				}
			}
			else
			{
				gAcc_Dir = 0;
				
				if( gAcc_Addr < 1024 )
				{
					gAcc_FuncBuf[gAcc_Addr >> 3] &= ~(1UL << (gAcc_Addr & 7));
				}
			}
			
			if( (gAcc_Addr > 0) && (gAcc_Addr <= 2044))
			{
				//セット
				ChangedTurnout(gAcc_Addr ,gAcc_Dir);
			}
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_AccCtrlMenu = 0;
			
			//パワーオフ
			ChangedPowerStatus(0);
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_UP) == true )
		{
			
			if( gAcc_Addr >= 1)
			{
				gAcc_Addr--;
			}
			else
			{
				gAcc_Addr = 1;
			}
			
			if( gAcc_Addr < 1024 )
			{
				gAcc_Dir = 1 & (gAcc_FuncBuf[gAcc_Addr >> 3] >> (gAcc_Addr & 7));
			}
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_DN) == true )
		{
			if( gAcc_Addr < 2044)
			{
				gAcc_Addr++;
			}
			else
			{
				gAcc_Addr = 2044;
			}
			
			if( gAcc_Addr < 1024 )
			{
				gAcc_Dir = 1 & (gAcc_FuncBuf[gAcc_Addr >> 3] >> (gAcc_Addr & 7));
			}
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else
		{
			
			if( editNums(inButtonStatus, &gAcc_Addr, 2044) == true)
			{
				
				if( gAcc_Addr < 1024 )
				{
					gAcc_Dir = 1 & (gAcc_FuncBuf[gAcc_Addr >> 3] >> (gAcc_Addr & 7));
				}
				
				//表示処理
				gDisplayFlag |= 1;
			}
		}
	}	
	
}



void DSSequence::interval_ErrorStatus(word inButtonStatus)
{

	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if(( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true ) ||
			( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true ))
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else
		{
			
		}
	}	
	
}


bool DSSequence::checkButton(word inBuf, word inCurrent, word inButtonBit)
{

	if( ((inBuf & inButtonBit) == 0) && ((inCurrent & inButtonBit) > 0))
	{
		return true;
	}
	else
	{
		return false;
	}
	
}

bool DSSequence::checkButtonLeaved(word inBuf, word inCurrent, word inButtonBit)
{

	if( ((inBuf & inButtonBit) > 0) && ((inCurrent & inButtonBit) == 0))
	{
		return true;
	}
	else
	{
		return false;
	}
	
}

bool DSSequence::checkButtonContinuous(word inBuf, word inCurrent, word inButtonBit)
{

	if( ((inBuf & inButtonBit) > 0) && ((inCurrent & inButtonBit) > 0))
	{
		return true;
	}
	else
	{
		return false;
	}
	
}

void DSSequence::StrCpyFromROM(char *outpText, const char *inpROMText, byte inLen)
{
	byte i;
	
	for( i = 0; i < inLen; i++)
	{
		outpText[i] = (char)pgm_read_byte(inpROMText + i);
	}
	
	for( i = inLen; i < 16; i++)
	{
		outpText[i] = 0x00;
	}
}


word DSSequence::ProcessCheck( byte inTempA, byte inTempB, byte inTempC)
{
	byte aChanged = 0;
	word aButtonData = 0;

// PCB Revision 4
	switch(inTempA)
	{
	case 1:
		//SW9, 5
		aButtonData |= BTN_ESC;//
		break;
	case 2:
		//SW2, 1
		aButtonData |= BTN_NUM2;//
		break;
	case 4:
		//SW13, 9
		aButtonData |= BTN_ENT;//
		break;
	case 8:
		//SW5, 4
		aButtonData |= BTN_NUM7;//
		break;
	case 16:
		//SW14, CLR
		aButtonData |= BTN_NUM8;//
		break;
	}
		
	switch(inTempB)
	{
	case 1:
		//SW1, 0
		aButtonData |= BTN_DN;//
		break;
	case 2:
		//SW3, 2
		aButtonData |= BTN_NUM3;//
		break;
	case 4:
		//SW12, 8
		aButtonData |= BTN_CLR;//
		break;
	case 8:
		//SW6, ESC
		aButtonData |= BTN_NUM6;//
		break;
	case 16:
		//SW15, UP
		aButtonData |= BTN_NUM0;//
		break;
	}

		
	switch(inTempC)
	{
	case 1:
		//SW10, 6
		aButtonData |= BTN_UP;//
		break;
	case 2:
		//SW4, 3
		aButtonData |= BTN_NUM4;//
		break;
	case 4:
		//SW11, 7
		aButtonData |= BTN_NUM1;//
		break;
	case 8:
		//SW7, ENTER
		aButtonData |= BTN_NUM5;//OK
		break;
	case 16:
		//SW8, DOWN
		aButtonData |= BTN_NUM9;//OK
		break;
	}
	
	return aButtonData;
	
}


byte DSSequence::GetButtonsFromAD(unsigned short inADData)
{
	/*
	R1	2.2kΩ
	R2	470Ω
	R3	1kΩ
	R4	2.2kΩ
	R5	6.8kΩ
	
	Threshold value:
	SW1: 82
	SW2: 290
	SW3: 525
	SW4: 747
	SW5: 942
	
	
	*/
	
	byte aRet = 0;
	
	if( inADData <= 82)
	{
		aRet = 1 << 0;
	}
	else if( inADData <= 290)
	{
		aRet = 1 << 1;
	}
	else if( inADData <= 525)
	{
		aRet = 1 << 2;
	}
	else if( inADData <= 747)
	{
		aRet = 1 << 3;
	}
	else if( inADData <= 942)
	{
		aRet = 1 << 4;
	}
	else
	{
		//Not pressed
	}
	
	
	return aRet;
}